@@ -2,16 +2,20 @@ import zmq
 from threading import Thread, Event
 import json
 import pandas as pd
-import numpy as np
 import os
 from time import sleep
 
+versionNo = "0.1.0"
+
 controlEndpoint = "tcp://127.0.0.1:5557"
 dataEndpoint = "tcp://127.0.0.1:5556"
 
 profilesPath = os.path.expanduser('~\\Documents\\Impulse\\Profiles\\')
 profilesPath = profilesPath.replace("\\","/")
 
+def version():
+    return versionNo
+
 def createControlMessage(details):
     controlString = json.dumps(details)
     controlStringB = bytes(controlString, 'utf_8')
@@ -55,7 +59,6 @@ def disconnect():
     sleep(1)
     print("Disconnected from impulse")
 
-
 def getStatus():
     controlDict = {
         '$type' : 'control.impulse.getStatus'
@@ -110,6 +113,7 @@ class heat():
             timeOrRate : rampTimeRate    
             }
         returnMessage = sendControl(controlDict)
+        self.busy = True
         return returnMessage["resultCode"]        
 
     def stopRamp(self):
@@ -147,6 +151,7 @@ class bias():
             'compliance' : compliance            
             }
         returnMessage = sendControl(controlDict)
+        self.busy = True
         return returnMessage["resultCode"]
 
     def stopSweepCycle(self):
@@ -160,6 +165,7 @@ class gas():
     def __init__(self):
         self.data = dataHandler("gas")
         self.msdata = dataHandler("massSpec")
+        self.busy = False
         self.gasSystemType = None
         self.controlMode = None
         
@@ -228,6 +234,7 @@ class gas():
             controlDict['gas3Flow']=gas3Flow
             controlDict['gas3FlowPath']=gas3FlowPath           
         returnMessage = sendControl(controlDict)
+        self.busy = True
         return returnMessage["resultCode"]
     
     def setIOP(self, inletPressure, outletPressure, gas1Flow=None, gas1FlowPath=None, gas2Flow=None, gas2FlowPath=None, gas3Flow=None, gas3FlowPath=None):
@@ -248,6 +255,7 @@ class gas():
             controlDict['gas1Concentration']=gas1Concentration
             controlDict['gas2Concentration']=gas2Concentration        
         returnMessage = sendControl(controlDict)
+        self.busy = True
         return returnMessage["resultCode"]    
     
     def setPF(self, reactorPressure, reactorFlow, gasConcentrationType=None, gas1Concentration=None, gas2Concentration=None):
@@ -296,12 +304,19 @@ class eventHandler():
             [address, contents] = self.eventSock.recv_multipart()
             contents = json.loads(contents)
             self.lastMssg = contents
-            if contents["$type"] == "data.event.profileController.stateChange":
+            if contents["$type"]=="data.event.stimulusActionFinished": #vergeten hoe die heette
+                if contents["stimulus"]=="heat":
+                    heat.busy = False
+                if contents["stimulus"]=="bias":
+                    bias.busy = False
+                if contents["stimulus"]=="gas":
+                    gas.busy = False
+            if contents["$type"] == "data.event.profileController.stateChanged": #deze moet ook anders heten
                 print("[PROFILE CONTROLLER]:",contents['state'])
-            elif contents["$type"] == "data.event.popup":
+            elif contents["$type"] == "data.event.popupRaised":
                 print("[IMPULSE POPUP]:",contents['message'])
-            elif contents["$type"] == "data.event.toast":
-                print("[IMPULSE TOAST]:",contents['message'])
+            elif contents["$type"] == "data.event.notificationRaised":
+                print("[IMPULSE NOTIFICATION]:",contents['message'])
             else:
                 print("[IMPULSE EVENT]:",contents)
         self.eventSock.setsockopt(zmq.LINGER, 0)
@@ -375,10 +390,48 @@ class dataHandler():
     def disconnect(self):
         self.event.set()
 
+
+    def roundDataFrame(self, dataSet):
+        decimalVals = {
+            'temperatureMeasured': 6,
+            'temperatureSetpoint': 6,
+            'powerMeasured': 3,
+            'currentMeasured': 18,
+            'voltageMeasured': 15,
+            'setpoint': 18,
+            'resistance': 11,
+            'compliance': 18,
+            'gas1FlowMeasured': 9,
+            'gas2FlowMeasured': 9,
+            'gas3FlowMeasured': 9,
+            'gas1ConcentrationMeasured': 2,
+            'gas2ConcentrationMeasured': 2,
+            'gas3ConcentrationMeasured': 2,
+            'gas1FlowSetpoint': 9,
+            'gas2FlowSetpoint': 9,
+            'gas3FlowSetpoint': 9,
+            'gas1ConcentrationSetpoint': 2,
+            'gas2ConcentrationSetpoint': 2,
+            'inletPressureMeasured': 7,
+            'outletPressureMeasured': 7,
+            'inletPressureSetpoint': 7,
+            'outletPressureSetpoint': 7,
+            'reactorPressureMeasured': 7,
+            'reactorFlowMeasured': 9,
+            'reactorPressureSetpoint': 7,
+            'reactorFlowSetpoint': 9,
+            'vacuumPressureMeasured': 9,
+            'flowCheckSetpoint': 9,
+            }
+        
+        dataSet = dataSet.round(decimalVals)
+        return dataSet
+        
     def getLastData(self):
         if self.dataSock is not None:
             self.lastSentSequence = self.lastMssg["sequenceNumber"]
-            return self.lastMssg
+            df = self.getDataFrame(-1)
+            return df
         else:
             print(f"[ALERT] Please subscribe using impulse.{self.topic}.data.subscribe() first")
             return []
@@ -388,7 +441,8 @@ class dataHandler():
             while self.lastMssg["sequenceNumber"] == self.lastSentSequence:
                 sleep(0.001)
             self.lastSentSequence = self.lastMssg["sequenceNumber"]
-            return self.lastMssg
+            df = self.getDataFrame(-1)
+            return df
         else:
             print(f"[ALERT] Please subscribe using impulse.{self.topic}.data.subscribe() first")
             return []
@@ -418,6 +472,7 @@ class dataHandler():
             df = self.data.iloc[startRowOrFlag:endRowOrFlag].copy()
             if len(df)>1:
                 df["timeStamp"] = pd.to_datetime(df['timeStamp']).dt.tz_convert(None)
+            df = self.roundDataFrame(df)
             return df
         else:
             print(f"[ALERT] Please subscribe using impulse.{self.topic}.data.subscribe() first")
@@ -427,4 +482,25 @@ profile = profile()
 heat = heat()
 bias = bias()
 gas = gas()
-events = eventHandler()
\ No newline at end of file
+events = eventHandler()
+
+
+heat.data.subscribe()
+bias.data.subscribe()
+gas.data.subscribe()
+gas.msdata.subscribe()
+
+heat.startRamp(500,'rampTime',10)
+sleep(5)
+print(heat.busy)
+
+lastdataHeat = heat.data.getLastData()
+lastDataBias = bias.data.getLastData()
+lastDataGas = gas.data.getLastData()
+lastDataMS = gas.msdata.getLastData()
+
+roundedHeat = heat.data.getDataFrame()
+roundedBias = bias.data.getDataFrame()
+roundedGas = gas.data.getDataFrame()
+roundedMS = gas.msdata.getDataFrame()
+
